// BenLincoln.DaVinci.GUI - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.GUI.

// BenLincoln.DaVinci.GUI is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci.GUI is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci.GUI (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci.GUI
{
    public enum ProcessableItemStatus
    {
        Processing,
        Queued,
        Error,
        Failed,
        Finished
    }

    public class ProcessableItem : IComparable
    {
        protected int _Position;
        protected string _Name;
        protected ProcessableItemStatus _Status;
        protected int _SuccessCount;
        protected int _FailureCount;
        protected string _Information;
        protected ProcessableItemCollection _ParentCollection;
        protected string _DefaultInformation;

        #region Properties

        public int Position
        {
            get
            {
                return _Position;
            }
            set
            {
                _Position = value;
            }
            //get
            //{
            //    if (ParentCollection == null)
            //    {
            //        return -1;
            //    }
            //    int pos = -1;
            //    try
            //    {
            //        pos = (ParentCollection.IndexOf(this) + 1);
            //    }
            //    catch (Exception ex)
            //    {
            //        pos = -1;
            //    }
            //    return pos;
            //}
        }

        public string Name
        {
            get
            {
                return _Name;
            }
            set
            {
                _Name = value;
            }
        }

        public ProcessableItemStatus Status
        {
            get
            {
                return _Status;
            }
            set
            {
                _Status = value;
            }
        }

        public int SuccessCount
        {
            get
            {
                return _SuccessCount;
            }
            set
            {
                _SuccessCount = value;
            }
        }

        public int FailureCount
        {
            get
            {
                return _FailureCount;
            }
            set
            {
                _FailureCount = value;
            }
        }

        public string Information
        {
            get
            {
                if (_Information == "")
                {
                    return _DefaultInformation;
                }
                return _Information;
            }
            set
            {
                _Information = value;
            }
        }

        public ProcessableItemCollection ParentCollection
        {
            get
            {
                return _ParentCollection;
            }
            set
            {
                _ParentCollection = value;
            }
        }

        #endregion

        public ProcessableItem()
        {
            Position = -1;
            Name = "";
            Status = ProcessableItemStatus.Queued;
            SuccessCount = 0;
            FailureCount = 0;
            Information = "";
            _DefaultInformation = "";
        }

        public ProcessableItem(ProcessableItem sourceItem)
            : this()
        {
            Position = sourceItem.Position;
            Name = sourceItem.Name;
            Status = sourceItem.Status;
            SuccessCount = sourceItem.SuccessCount;
            FailureCount = sourceItem.FailureCount;
            Information = sourceItem.Information;
        }

        public ProcessableItem(string name)
            : this()
        {
            Name = name;
        }

        public ProcessableItem(string name, ProcessableItemStatus status, string information)
            : this()
        {
            Name = name;
            Status = status;
            Information = information;
        }

        ~ProcessableItem()
        {
            _ParentCollection = null;
        }

        public int CompareTo(object obj)
        {
            if (obj is ProcessableItem)
            {
                ProcessableItem b2 = (ProcessableItem)obj;
                return Name.CompareTo(b2.Name);
            }
            else
            {
                return Name.CompareTo(obj.ToString());
            }
        }

        public void SetDefaultInformation(string text)
        {
            _DefaultInformation = text;
        }
    }
}
