// BenLincoln.DaVinci.GUI - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.GUI.

// BenLincoln.DaVinci.GUI is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci.GUI is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci.GUI (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.Common;
using System.Drawing;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;

using BenLincoln.Utility;

namespace BenLincoln.DaVinci.GUI
{


    public partial class ProcessableItemQueueGrid : UserControl
    {
        public enum SortColumn
        {
            Position,
            Name,
            Status,
            Information
        }

        protected const int _UpdateTimerInterval = 1000;

        protected string _ItemTypeNameSingular;
        protected string _ItemTypeNamePlural;

        protected bool _AddItemsRecursively;

        protected string _LastUsedDirectory;

        //protected DataTable _QueueData;

        protected ProcessableItemCollection _Items;

        protected System.Windows.Forms.Timer _UpdateTimer;

        protected string _SortColumnName;
        protected ListSortDirection _SortColumnDirection;

        //public delegate void WaitStartedHandler(object sender, EventArgs e);
        //public delegate void WaitFinishedHandler(object sender, EventArgs e);

        public event System.EventHandler WaitStarted;
        public event System.EventHandler WaitFinished;

        protected System.EventHandler _WaitStartedHandler;
        protected System.EventHandler _WaitFinishedHandler;

        #region Properties

        public string ItemTypeNameSingular
        {
            get
            {
                return _ItemTypeNameSingular;
            }
            set
            {
                _ItemTypeNameSingular = value;
                UpdateLabels();
            }
        }

        public string ItemTypeNamePlural
        {
            get
            {
                return _ItemTypeNamePlural;
            }
            set
            {
                _ItemTypeNamePlural = value;
                UpdateLabels();
            }
        }

        public bool AddItemsRecursively
        {
            get
            {
                return _AddItemsRecursively;
            }
            set
            {
                _AddItemsRecursively = value;
                chkAddRecursively.Checked = value;
            }
        }

        public ProcessableItemCollection Items
        {
            get
            {
                return _Items;
            }
            set
            {
                _Items = value;
                BindItems();
                ForceQueueUpdate();
            }
        }

        //public DataTable QueueData
        //{
        //    get
        //    {
        //        return _QueueData;
        //    }
        //    set
        //    {
        //        _QueueData = value;
        //    }
        //}

        public bool ShowAddRecursivelyCheckbox
        {
            get
            {
                return pnlControls2.Visible;
            }
            set
            {
                pnlControls2.Visible = value;
            }
        }

        public bool ShowControlButtons
        {
            get
            {
                return pnlControls1.Visible;
            }
            set
            {
                pnlControls1.Visible = value;
            }
        }

        public bool EnableButtonAddItem
        {
            get
            {
                return btnAdd.Enabled;
            }
            set
            {
                btnAdd.Enabled = value;
            }
        }

        public bool EnableButtonAddItemsRecursively
        {
            get
            {
                return btnAddRecursively.Enabled;
            }
            set
            {
                btnAddRecursively.Enabled = value;
            }
        }

        public bool EnableButtonRemoveSelected
        {
            get
            {
                return btnRemoveSelected.Enabled;
            }
            set
            {
                btnRemoveSelected.Enabled = value;
            }
        }

        public bool EnableButtonRemoveCompleted
        {
            get
            {
                return btnRemoveCompleted.Enabled;
            }
            set
            {
                btnRemoveCompleted.Enabled = value;
            }
        }

        public bool EnableDragAndDrop
        {
            get
            {
                return dgQueue.AllowDrop;
            }
            set
            {
                dgQueue.AllowDrop = value;
            }
        }

        //public DragEventHandler DragDrop
        //{
        //    get
        //    {
        //        return dgQueue.DragDrop;
        //    }
        //    set
        //    {
        //        dgQueue.DragDrop = value;
        //    }
        //}

        #endregion

        public ProcessableItemQueueGrid(string itemTypeNameSingular, string itemTypeNamePlural)
        {
            InitializeComponent();
            _LastUsedDirectory = "";
            dgQueue.Dock = DockStyle.Fill;
            dgQueue.CellFormatting += QueueGrid_CellFormatting;

            _ItemTypeNameSingular = itemTypeNameSingular;
            _ItemTypeNamePlural = itemTypeNamePlural;

            //_SortColumnName = "Position";
            //_SortColumnDirection = ListSortDirection.Ascending;

            //InitializeQueueTable();
            Items = new ProcessableItemCollection();
            //BindItems();
            //dgQueue.Columns.Clear();
            //dgQueue.DataSource = Items;

            //RemoveUnwantedColumns();

            //dgQueue.DataBindings.CollectionChanged += new CollectionChangeEventHandler(DataBindings_CollectionChanged);
            //Items.ListChanged += new ListChangedEventHandler(Items_ListChanged);

            //ProcessableItem pi = new ProcessableItem("Test item 1", ProcessableItemStatus.Queued, "");
            //Items.Add(pi);

            //pi = new ProcessableItem("Test item 2", ProcessableItemStatus.Processing, "");
            //Items.Add(pi);

            //pi = new ProcessableItem("Test item 3", ProcessableItemStatus.Finished, "Generated 174 images.");
            //Items.Add(pi);

            //pi = new ProcessableItem("Test item 4", ProcessableItemStatus.Error, "Generated 14 errors.");
            //Items.Add(pi);

            //InitializeQueue();

            BuildContextMenus();

            //dgQueue.DataSource = QueueData;
            //dgQueue.DataBindings.Add(

            //dgQueue.Columns

            _UpdateTimer = new System.Windows.Forms.Timer();
            _UpdateTimer.Interval = _UpdateTimerInterval;
            _UpdateTimer.Tick += _UpdateTimer_Tick;
            _UpdateTimer.Enabled = true;
        }

        public void SetWaitStartedHandler(System.EventHandler newHandler)
        {
            RemoveWaitStartedHandler();
            _WaitStartedHandler = newHandler;
            WaitStarted += _WaitStartedHandler;
        }

        public void SetWaitFinishedHandler(System.EventHandler newHandler)
        {
            RemoveWaitFinishedHandler();
            _WaitFinishedHandler = newHandler;
            WaitFinished += _WaitFinishedHandler;
        }

        protected void RemoveWaitStartedHandler()
        {
            if (_WaitStartedHandler != null)
            {
                WaitStarted -= _WaitStartedHandler;
                _WaitStartedHandler = null;
            }
        }

        protected void RemoveWaitFinishedHandler()
        {
            if (_WaitFinishedHandler != null)
            {
                WaitFinished -= _WaitFinishedHandler;
                _WaitFinishedHandler = null;
            }
        }

        protected void TriggerWaitStarted()
        {
            if (WaitStarted != null)
            {
                WaitStarted(this, new EventArgs());
            }
        }

        protected void TriggerWaitFinished()
        {
            if (WaitFinished != null)
            {
                
                WaitFinished(this, new EventArgs());
            }
        }

        protected void BindItems()
        {
            dgQueue.Columns.Clear();
            dgQueue.DataSource = Items;

            dgQueue.DataBindings.CollectionChanged += new CollectionChangeEventHandler(DataBindings_CollectionChanged);
            Items.ListChanged += new ListChangedEventHandler(Items_ListChanged);

            InitializeQueue();
        }

        void _UpdateTimer_Tick(object sender, EventArgs e)
        {
            //BindItems();
            //SetSortColumn(_SortColumnName, _SortColumnDirection);

            dgQueue.Refresh();
        }

        //public SortColumn CurrentSortColumn
        //{
        //    get
        //    {
        //        if (dgQueue.SortedColumn != null)
        //        {
        //            if (dgQueue.SortedColumn.Name == "Name")
        //            {
        //                return SortColumn.Name;
        //            }
        //            if (dgQueue.SortedColumn.Name == "Status")
        //            {
        //                return SortColumn.Status;
        //            }
        //            if (dgQueue.SortedColumn.Name == "Information")
        //            {
        //                return SortColumn.Information;
        //            }
        //        }
        //        return SortColumn.Position;
        //    }
        //}

        //public ListSortDirection CurrentSortDirection
        //{
        //    get
        //    {
        //        if (dgQueue.SortedColumn != null)
        //        {
        //            if (dgQueue.SortedColumn.Name == "Name")
        //            {
        //                return SortColumn.Name;
        //            }
        //            if (dgQueue.SortedColumn.Name == "Status")
        //            {
        //                return SortColumn.Status;
        //            }
        //            if (dgQueue.SortedColumn.Name == "Information")
        //            {
        //                return SortColumn.Information;
        //            }
        //        }
        //        return SortColumn.Position;
        //    }
        //}

        public void ForceQueueUpdate()
        {
            if (dgQueue.SortedColumn != null)
            {
                if (dgQueue.SortOrder == SortOrder.Descending)
                {
                    SetSortColumn(dgQueue.SortedColumn.Name, ListSortDirection.Descending);
                }
                else
                {
                    SetSortColumn(dgQueue.SortedColumn.Name, ListSortDirection.Ascending);
                }
            }
            else
            {
                SetSortColumn("Position", ListSortDirection.Ascending);
            }
        }

        public ProcessableItemQueueGrid()
            : this("Directory", "Directories")
        {
        }

        protected void UpdateLabels()
        {
            //RemoveUnwantedColumns();

            if (dgQueue.Columns.Contains("SuccessCount"))
            {
                dgQueue.Columns["SuccessCount"].Visible = false;
            }
            if (dgQueue.Columns.Contains("FailureCount"))
            {
                dgQueue.Columns["FailureCount"].Visible = false;
            }

            if (dgQueue.Columns.Contains("Position"))
            {
                dgQueue.Columns["Position"].HeaderText = "#";
            }
            if (dgQueue.Columns.Contains("Name"))
            {
                dgQueue.Columns["Name"].HeaderText = ItemTypeNameSingular;
            }
            btnAdd.Text = "Add " + ItemTypeNameSingular;
            //btnAddRecursively.Text = "Add " + ItemTypeNamePlural + " Recursively";
            btnAddRecursively.Text = "Add Recursively";
            btnRemoveSelected.Text = "Remove Selected " + ItemTypeNamePlural;
            btnRemoveCompleted.Text = "Remove Completed " + ItemTypeNamePlural;
            chkAddRecursively.Text = "Add " + ItemTypeNamePlural + " Recursively When Dropped Onto The Queue";

            UpdateItemCountLabel();
           
            UpdateColumnWidths();
        }

        protected void UpdateItemCountLabel()
        {
            if (Items.Count == 1)
            {
                lblItemCount.Text = "1 " + ItemTypeNameSingular;
            }
            else
            {
                lblItemCount.Text = Items.Count.ToString() + " " + ItemTypeNamePlural;
            }
        }

        protected void UpdateColumnWidths()
        {
            if (dgQueue.Columns.Contains("Position"))
            {
                dgQueue.Columns["Position"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            }
            if (dgQueue.Columns.Contains("Status"))
            {
                dgQueue.Columns["Status"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            }
            if (dgQueue.Columns.Contains("Information"))
            {
                dgQueue.Columns["Information"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            }
        }

        protected void QueueGrid_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            if (e.RowIndex < Items.Count)
            {
                Color newColour = SystemColors.ControlText;
                bool bold = false;

                switch (Items[e.RowIndex].Status)
                {
                    case ProcessableItemStatus.Processing:
                        bold = true;
                        break;
                    case ProcessableItemStatus.Queued:
                        break;
                    case ProcessableItemStatus.Error:
                        bold = true;
                        newColour = Color.FromArgb(92, 0, 0);
                        break;
                    case ProcessableItemStatus.Finished:
                        newColour = Color.FromArgb(0, 92, 0);
                        break;
                    case ProcessableItemStatus.Failed:
                        bold = true;
                        newColour = Color.FromArgb(92, 0, 0);
                        break;
                    default:
                        //newColour = Color.FromArgb(255, 0, 255);
                        break;
                }
                for (int j = 0; j < dgQueue.Rows[e.RowIndex].Cells.Count; j++)
                {
                    dgQueue.Rows[e.RowIndex].Cells[j].Style.ForeColor = newColour;
                    Font currentFont = SystemFonts.DefaultFont;
                    if (bold)
                    {
                        dgQueue.Rows[e.RowIndex].Cells[j].Style.Font =
                            new Font(currentFont,
                            FontStyle.Bold);
                    }
                    else
                    {
                        dgQueue.Rows[e.RowIndex].Cells[j].Style.Font =
                            new Font(currentFont,
                            FontStyle.Regular);
                    }
                }
            }
        }

        protected void BuildContextMenus()
        {
            ContextMenu cmQueue = new ContextMenu();
            cmQueue.MenuItems.Add("Copy Selected " + _ItemTypeNameSingular + " To Clipboard", ContextMenuCopyName_Click);
            cmQueue.MenuItems.Add("Copy Selected Information To Clipboard", ContextMenuCopyInformation_Click);
            cmQueue.MenuItems.Add("Copy All Data To Clipboard", ContextMenuCopyAllData_Click);
            this.ContextMenu = cmQueue;
        }


        public void SortBy(SortColumn column)
        {
            SortByFunction(column);
        }

        delegate void SortByCallback(SortColumn column);

        protected void SortByFunction(SortColumn column)
        {
            if (this.InvokeRequired)
            {
                SortByCallback c = new SortByCallback(SortByFunction);
                this.Invoke(c, new object[] { column });
            }
            else
            {
                switch (column)
                {
                    case SortColumn.Position:
                        //_SortColumnName = "Position";
                        //_SortColumnDirection = ListSortDirection.Ascending;
                        SetSortColumn("Position", ListSortDirection.Ascending);
                        break;
                    case SortColumn.Name:
                        //_SortColumnName = "Name";
                        //_SortColumnDirection = ListSortDirection.Ascending;
                        SetSortColumn("Name", ListSortDirection.Ascending);
                        break;
                    case SortColumn.Status:
                        //_SortColumnName = "Status";
                        //_SortColumnDirection = ListSortDirection.Ascending;
                        SetSortColumn("Status", ListSortDirection.Ascending);
                        break;
                    case SortColumn.Information:
                        //_SortColumnName = "Information";
                        //_SortColumnDirection = ListSortDirection.Ascending;
                        SetSortColumn("Information", ListSortDirection.Ascending);
                        break;
                }
            }
        }

        protected void InitializeQueue()
        {
            //dgQueue.Columns.Clear();
            //dgQueue.DataSource = null;
            //dgQueue.DataSource = Items;
            dgQueue.DataBindings.DefaultDataSourceUpdateMode = DataSourceUpdateMode.OnPropertyChanged;

            //dgQueue.Sort(dgQueue.Columns["Status"], ListSortDirection.Ascending);

            UpdateLabels();
        }

        protected void SetSortColumn(string columnName, ListSortDirection direction)
        {
            if (dgQueue.Columns.Contains(columnName))
            {
                try
                {
                    dgQueue.Sort(dgQueue.Columns[columnName], direction);
                }
                catch (Exception ex)
                {
                    // this is not a critical error, so fail silently
                }
            }
        }

        public void CopyCurrentNameToClipboard()
        {
            string resultText = GetSelectedValues("Name");
            if (resultText.Trim() != "")
            {
                Clipboard.SetText(resultText);
            }
        }

        public void CopyCurrentInformationToClipboard()
        {
            string resultText = GetSelectedValues("Information");
            if (resultText.Trim() != "")
            {
                Clipboard.SetText(resultText);
            }
        }

        public void CopyAllInformationToClipboard()
        {
            string resultText = GetAllValues();
            if (resultText.Trim() != "")
            {
                Clipboard.SetText(resultText);
            }
        }

        public void RemoveSelectedItems()
        {
            DataGridViewSelectedRowCollection selected = dgQueue.SelectedRows;
            for (int i = 0; i < selected.Count; i++)
            {
                dgQueue.Rows.Remove(selected[i]);
            }
            UpdateLabels();
        }

        public void RemoveCompletedItems()
        {
            for (int i = 0; i < Items.Count; i++)
            {
                if (Items[i].Status == ProcessableItemStatus.Finished)
                {
                    Items.RemoveAt(i);
                    i--;
                }
            }
            UpdateLabels();
        }

        public void MoveSelectedItemsUp()
        {
            string[] positions = GetSelectedValuesArray("Position");
            Array.Sort(positions);
            string[] selectedNames = GetSelectedValuesArray("Name");

            int[] selectedPositions = new int[positions.Length];

            for (int i = 0; i < positions.Length; i++)
            {
                int pos = int.Parse(positions[i]);
                selectedPositions[i] = pos;
                if (pos > 1)
                {
                    //Items.Swap(pos, (pos - 1));
                    //selectedPositions[i] = pos - 1;
                    int targetIndex = Items.IndexOf(pos - 1);
                    if (targetIndex >= 0)
                    {
                        if (!ItemIsSelected(selectedNames, Items[targetIndex].Name))
                        {
                            Items.Swap(pos, (pos - 1));
                            selectedPositions[i] = pos - 1;
                        }
                    }
                }
            }

            RefreshAfterMove(selectedNames);

            UpdateLabels();
        }

        public void MoveSelectedItemsDown()
        {
            string[] positions = GetSelectedValuesArray("Position");
            Array.Sort(positions);
            string[] selectedNames = GetSelectedValuesArray("Name");
            int[] selectedPositions = new int[positions.Length];

            //for (int i = 0; i < positions.Length; i++)
            for (int i = (positions.Length - 1); i >= 0; i--)
            {
                int pos = int.Parse(positions[i]);
                selectedPositions[i] = pos;
                if (pos < (Items.Count))
                {
                    int targetIndex = Items.IndexOf(pos + 1);
                    if (targetIndex >= 0)
                    {
                        if (!ItemIsSelected(selectedNames, Items[targetIndex].Name))
                        {
                            Items.Swap(pos, (pos + 1));
                            selectedPositions[i] = pos + 1;
                        }
                    }
                }
            }

            RefreshAfterMove(selectedNames);

            UpdateLabels();
        }

        protected bool ItemIsSelected(string[] selectedNames, string name)
        {
            foreach (string cName in selectedNames)
            {
                if (cName == name)
                {
                    return true;
                }
            }
            return false;
        }

        protected void RefreshAfterMove(string[] selectedNames)
        {
            dgQueue.Refresh();

            //RemoveUnwantedColumns();

            if ((dgQueue.Columns.Contains("Position")) && (dgQueue.Columns.Contains("Name")))
            {
                dgQueue.Sort(dgQueue.Columns["Position"], ListSortDirection.Ascending);

                for (int j = 0; j < dgQueue.Rows.Count; j++)
                {
                    dgQueue.Rows[j].Selected = false;
                }

                for (int i = 0; i < selectedNames.Length; i++)
                {
                    for (int j = 0; j < dgQueue.Rows.Count; j++)
                    {
                        if (dgQueue.Rows[j].Cells["Name"].Value.ToString() == selectedNames[i].ToString())
                        {
                            dgQueue.Rows[j].Selected = true;
                        }
                    }
                }

                dgQueue.Sort(dgQueue.Columns["Position"], ListSortDirection.Ascending);
            }



            dgQueue.Refresh();

            //RemoveUnwantedColumns();
        }

        protected void ContextMenuCopyName_Click(object sender, System.EventArgs e)
        {
            CopyCurrentNameToClipboard();
        }

        protected void ContextMenuCopyInformation_Click(object sender, System.EventArgs e)
        {
            CopyCurrentInformationToClipboard();
        }

        protected void ContextMenuCopyAllData_Click(object sender, System.EventArgs e)
        {
            CopyAllInformationToClipboard();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            DisplayAddDirectoryDialogue(false);
        }

        private void btnAddRecursively_Click(object sender, EventArgs e)
        {
            DisplayAddDirectoryDialogue(true);
        }

        private void btnRemoveSelected_Click(object sender, EventArgs e)
        {
            RemoveSelectedItems();
        }

        private void btnRemoveCompleted_Click(object sender, EventArgs e)
        {
            RemoveCompletedItems();
        }

        private void btnMoveUp_Click(object sender, EventArgs e)
        {
            MoveSelectedItemsUp();
        }

        private void btnMoveDown_Click(object sender, EventArgs e)
        {
            MoveSelectedItemsDown();
        }

        protected string GetSelectedValues(string columnName)
        {
            StringBuilder builder = new StringBuilder();

            //for (int i = 0; i < dgQueue.SelectedRows.Count; i++)
            //{
            //    builder.Append(dgQueue.SelectedRows[i].Cells[columnName].Value.ToString());
            //    if (i < (dgQueue.SelectedRows.Count - 1))
            //    {
            //        builder.Append(Environment.NewLine);
            //    }
            //}

            string[] values = GetSelectedValuesArray(columnName);

            for (int i = 0; i < values.Length; i++)
            {
                builder.Append(values[i]);
                if (i < (values.Length - 1))
                {
                    builder.Append(Environment.NewLine);
                }
            }

            return builder.ToString();
        }

        protected string[] GetSelectedValuesArray(string columnName)
        {
            string[] result = new string[dgQueue.SelectedRows.Count];

            for (int i = 0; i < dgQueue.SelectedRows.Count; i++)
            {
                result[i] = (dgQueue.SelectedRows[i].Cells[columnName].Value.ToString());
            }

            return result;
        }

        protected string GetSelectedValues()
        {
            StringBuilder builder = new StringBuilder();

            for (int i = 0; i < dgQueue.SelectedRows.Count; i++)
            {
                for (int j = 0; j < dgQueue.SelectedRows[i].Cells.Count; j++)
                {
                    builder.Append(dgQueue.SelectedRows[i].Cells[j].Value.ToString());
                    if (j < (dgQueue.SelectedRows[i].Cells.Count - 1))
                    {
                        builder.Append("\t");
                    }
                }
                if (i < (dgQueue.SelectedRows.Count - 1))
                {
                    builder.Append(Environment.NewLine);
                }
            }

            return builder.ToString();
        }

        protected string GetAllValues()
        {
            StringBuilder builder = new StringBuilder();

            for (int i = 0; i < dgQueue.Rows.Count; i++)
            {
                for (int j = 0; j < dgQueue.Rows[i].Cells.Count; j++)
                {
                    builder.Append(dgQueue.Rows[i].Cells[j].Value.ToString());
                    if (j < (dgQueue.Rows[i].Cells.Count - 1))
                    {
                        builder.Append("\t");
                    }
                }
                if (i < (dgQueue.Rows.Count - 1))
                {
                    builder.Append(Environment.NewLine);
                }
            }

            return builder.ToString();
        }

        private void dgQueue_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Copy;
            }
        }

        private void dgQueue_DragDrop(object sender, DragEventArgs e)
        {
            string[] pathList = (string[])e.Data.GetData(DataFormats.FileDrop);

            AddDirectories(pathList, AddItemsRecursively);
        }

        public void DisplayAddDirectoryDialogue(bool addRecursively)
        {
            FolderBrowserDialog fd = new FolderBrowserDialog();
            if (_LastUsedDirectory != "")
            {
                if (Directory.Exists(_LastUsedDirectory))
                {
                    fd.SelectedPath = _LastUsedDirectory;
                }
            }
            fd.Description = "Select a " + ItemTypeNameSingular +
                " to add to the queue";
            DialogResult r = fd.ShowDialog();
            if ((r == DialogResult.OK) && (fd.SelectedPath.Trim() != ""))
            {
                _LastUsedDirectory = fd.SelectedPath;
                AddDirectories(new string[] { fd.SelectedPath }, addRecursively);
            }
        }

        public void AddDirectories(string[] directories, bool addRecursively)
        {
            //TriggerWaitStarted();
            Thread wsThread = new Thread(new ThreadStart(TriggerWaitStarted));
            wsThread.Start();

            try
            {
                string[] pathList = directories;

                if (addRecursively)
                {
                    pathList = GetRecursiveDirectoryList(pathList);
                }

                foreach (string path in pathList)
                {
                    if (Directory.Exists(path))
                    {
                        if (!Items.Contains(path))
                        {
                            Items.Add(new ProcessableItem(path, ProcessableItemStatus.Queued, ""));
                        }
                    }
                }

                InitializeQueue();
            }
            catch (Exception ex)
            {
                throw new Exception("An error occurred adding the selected directories.", ex);
                TriggerWaitFinished();
            }
            finally
            {
                TriggerWaitFinished();
            }            
        }

        protected string[] GetRecursiveDirectoryList(string[] baseDirectories)
        {
            DirectoryLister dl = new DirectoryLister();
            string[] errors = new string[0];
            string[] result = dl.GetRecursiveDirectoryList(baseDirectories, out errors);
            if (errors.Length > 0)
            {
                MessageBox.Show("Warning: " + errors.Length.ToString() +
                    " errors occurred while attempting to add directories recursively. " +
                    " Some of the content will not be added to the list.", "I/O Error", 
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return result;
        }

        protected void chkAddRecursively_CheckedChanged(object sender, EventArgs e)
        {
            AddItemsRecursively = chkAddRecursively.Checked;
        }

        protected void DataBindings_CollectionChanged(object sender, CollectionChangeEventArgs e)
        {
            UpdateLabels();
            this.Invalidate();
        }

        void Items_ListChanged(object sender, ListChangedEventArgs e)
        {
            UpdateLabels();
            this.Invalidate();
        }


    }
}
