// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace BenLincoln.DaVinci
{
    public enum CustomGreyscaleChannelType
    {
        None,
        BandCategory,
        CustomScriptCode
    }

    public class CustomGreyscaleChannelConfig
    {
        protected string _ChannelName;
        protected CustomGreyscaleChannelType _ChannelType;
        protected BandCategory _Category;
        protected string _CustomScriptCode;
        protected BandCombineMethod _BandCombineMode;
        protected bool _UseAsLuminanceChannel;
        protected bool _UseAsThreeChannelChannel;
        //protected NormalizationMode _OutputNormalization;
        protected TransformationProfileCollection _InputTransformations;
        protected TransformationProfileCollection _OutputTransformations;

        #region Properties

        public string ChannelName
        {
            get
            {
                return _ChannelName;
            }
            set
            {
                _ChannelName = value;
            }
        }

        public CustomGreyscaleChannelType ChannelType
        {
            get
            {
                return _ChannelType;
            }
            set
            {
                _ChannelType = value;
            }
        }

        public BandCategory Category
        {
            get
            {
                return _Category;
            }
            set
            {
                _Category = value;
            }
        }

        public string CustomScriptCode
        {
            get
            {
                return _CustomScriptCode;
            }
            set
            {
                _CustomScriptCode = value;
            }
        }

        public BandCombineMethod BandCombineMode
        {
            get
            {
                return _BandCombineMode;
            }
            set
            {
                _BandCombineMode = value;
            }
        }

        public bool UseAsLuminanceChannel
        {
            get
            {
                return _UseAsLuminanceChannel;
            }
            set
            {
                _UseAsLuminanceChannel = value;
            }
        }

        public bool UseAsThreeChannelChannel
        {
            get
            {
                return _UseAsThreeChannelChannel;
            }
            set
            {
                _UseAsThreeChannelChannel = value;
            }
        }

        //public NormalizationMode OutputNormalization
        //{
        //    get
        //    {
        //        return _OutputNormalization;
        //    }
        //    set
        //    {
        //        _OutputNormalization = value;
        //    }
        //}

        public TransformationProfileCollection InputTransformations
        {
            get
            {
                return _InputTransformations;
            }
            set
            {
                _InputTransformations = value;
            }
        }

        public TransformationProfileCollection OutputTransformations
        {
            get
            {
                return _OutputTransformations;
            }
            set
            {
                _OutputTransformations = value;
            }
        }

        #endregion

        public CustomGreyscaleChannelConfig()
        {
            _ChannelName = "";
            _ChannelType = CustomGreyscaleChannelType.None;
            _Category = BandCategory.None;
            _CustomScriptCode = "";
            _BandCombineMode = BandCombineMethod.Average;
            _UseAsLuminanceChannel = false;
            _UseAsThreeChannelChannel = false;
            //_OutputNormalization = NormalizationMode.None;
            InputTransformations = new TransformationProfileCollection();
            OutputTransformations = new TransformationProfileCollection();
        }

        public bool IsValidConfiguration()
        {
            if (ChannelName == "")
            {
                return false;
            }
            switch (ChannelType)
            {
                case CustomGreyscaleChannelType.None:
                    return false;
                    break;
                case CustomGreyscaleChannelType.BandCategory:
                    if (Category != BandCategory.None)
                    {
                        return true;
                    }
                    break;
                case CustomGreyscaleChannelType.CustomScriptCode:
                    if (CustomScriptCode != "")
                    {
                        return true;
                    }
                    break;
            }
            return false;
        }

        public static CustomGreyscaleChannelType CustomGreyscaleChannelTypeFromString(string channelType)
        {
            string cType = channelType.Trim().ToLower();

            switch (cType)
            {
                case "bandcategory":
                    return CustomGreyscaleChannelType.BandCategory;
                    break;
                case "customscriptcode":
                    return CustomGreyscaleChannelType.CustomScriptCode;
                    break;
            }
            throw new ArgumentException("Could not convert the string '" + channelType + "' to a recognized custom greyscale channel type.");
        }
    }
}
