// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class MultiChannelImage : OutputImage
    {
        protected GreyscaleChannel[] _Channels;

        //protected const string _FileOutputDirectory = "";
        //protected const string _FileExtension = ".TIF";
        //protected const string _FileDVType = "tiff";
        //protected const string _FileDVOverwrite = "1";

        //protected OutputParameterSet[] _OutputFormats;

        //protected bool RegenerateExistingImages;

        //protected bool _WillOverwriteExistingData;

        //protected bool _WillBeCreatedByScript;

        protected string _TemporaryObjectNamePrefix = "mci_temp_";
        // the script code to insert when an object should be removed from memory
        //protected const string _DeallocateObjectScriptCode = " = 0";

        #region Properties

        public GreyscaleChannel[] Channels
        {
            get
            {
                return _Channels;
            }
            set
            {
                _Channels = value;
                GenerateName();
            }
        }

        //public OutputParameterSet[] OutputFormats
        //{
        //    get
        //    {
        //        return _OutputFormats;
        //    }
        //    set
        //    {
        //        _OutputFormats = value;
        //    }
        //}

        /// <summary>
        /// If this value is true, then any three-colour image files will be 
        /// overwritten as if they did not already exist. If this value is false, then
        /// the generated script will not recreate them. Defaults to false to avoid 
        /// unnecessary processing.
        /// </summary>
        //public bool regenerateExistingFiles
        //{
        //    get
        //    {
        //        return RegenerateExistingImages;
        //    }
        //    set
        //    {
        //        RegenerateExistingImages = value;
        //    }
        //}

        ///// <summary>
        ///// This read-only value represents whether or not running the script which represents
        ///// the image will overwrite data which already exists on disk.
        ///// </summary>
        //public bool WillOverwriteExistingData
        //{
        //    get
        //    {
        //        return _WillOverwriteExistingData;
        //    }
        //}

        ///// <summary>
        ///// This read-only value represents whether or not running the script which represents
        ///// the image will create a file
        ///// </summary>
        //public bool WillBeCreatedByScript
        //{
        //    get
        //    {
        //        return _WillBeCreatedByScript;
        //    }
        //}

        ///// <summary>
        ///// Controls whether or not the image will be used as the basis for second-order images
        ///// (Luma/Colour, etc.)
        ///// </summary>
        //public bool IncludeInSecondOrderImages
        //{
        //    get
        //    {
        //        return _IncludeInSecondOrderImages;
        //    }
        //    set
        //    {
        //        _IncludeInSecondOrderImages = value;
        //    }
        //}

        #endregion

        public MultiChannelImage()
            : base()
        {
            //RegenerateExistingImages = false;
            //RegenerateExistingImages = true;
            //_WillOverwriteExistingData = false;
            //_WillBeCreatedByScript = false;
            _IncludeInSecondOrderImages = true;
            _NamePrefix = "FC-";
            _TemporaryObjectNamePrefix = "mci_temp_";
            //_OutputFormats = new OutputParameterSet[1];
            //_OutputFormats[0] = new OutputParameterSet();
            //_OutputFormats[0].BitsPerChannel = BitsPerChannel.SixteenShort;
        }

        //public virtual string GetFilePath()
        //{
        //    StringBuilder builder = new StringBuilder(_FileOutputDirectory);
        //    builder.Append(_ImageName);
        //    //builder.Append(_FileExtension);
        //    return builder.ToString();
        //}

        public virtual string GetFilePathWithoutExtension()
        {
            StringBuilder builder = new StringBuilder(OutputDirectory);
            builder.Append(_ImageName);
            return builder.ToString();
        }

        public virtual string GetFilePathWithPrimaryExtension()
        {
            if (OutputFormats.Length == 0)
            {
                throw new Exception("There are no output formats to obtain a file extension from.");
            }
            StringBuilder builder = new StringBuilder(OutputDirectory);
            builder.Append(_ImageName);
            builder.Append(OutputParameterSet.ImageFormatToFileExtension(OutputFormats[0].FileFormat));
            return builder.ToString();
        }

        public virtual bool ContainsBand(string bandName)
        {
            foreach (GreyscaleChannel channel in _Channels)
            {
                if (channel.ContainsBand(bandName))
                {
                    return true;
                }
            }

            return false;
        }

        public virtual bool ContainsBand(SpectralBand band)
        {
            return ContainsBand(band.BandName);
        }

        public override string GetName()
        {
            return ImageName;
        }

        protected override string _GetReadScriptCode(string scriptPrepend, string scriptTargetObjectName)
        {
            return _GetReadScriptCode(scriptPrepend, scriptTargetObjectName, GetFilePathWithPrimaryExtension());
        }

        protected string _GetReadScriptCode(string scriptPrepend, string scriptTargetObjectName, string filePath)
        {
            StringBuilder builder = new StringBuilder(scriptPrepend);

            builder.Append(scriptTargetObjectName);
            builder.Append(" = ");
            builder.Append(_CastOnLoad);
            builder.Append("(");
            builder.Append(_DataOrganizationMode);
            builder.Append("(read(\"");
            builder.Append(filePath);
            builder.Append("\")))");
            builder.Append(Environment.NewLine);
            return builder.ToString();
        }

        protected virtual string GetImageFileWriteCode(string scriptTargetObjectName, bool regenerateExistingFiles)
        {
            StringBuilder builder = new StringBuilder();

            // normalize the image if that option is enabled
            //builder.Append(WrapScriptObjectInNormalizationCode(scriptTargetObjectName, OutputNormalization));
            //builder.Append(GetFullNormalizationWrappedLine(scriptTargetObjectName, OutputNormalization));
            //builder.Append(OutputTransformations.GetWrappedLine(scriptTargetObjectName));

            builder.Append(GetImageFileWriteCode(scriptTargetObjectName, OutputFormats, GetFilePathWithoutExtension(), regenerateExistingFiles));

            return builder.ToString();
        }

        /// <summary>
        /// Adds a new channel to the multi-channel image - this is currently only used for 
        /// keeping track of which spectral bands make up an image
        /// </summary>
        /// <param name="newChannel">the GreyScaleChannel to add</param>
        public void AddChannel(GreyscaleChannel newChannel)
        {
            if (_Channels == null)
            {
                _Channels = new GreyscaleChannel[1];
            }
            else
            {
                Array.Resize(ref _Channels, _Channels.Length + 1);
            }
            _Channels[_Channels.Length - 1] = newChannel;
        }

        public override int GetMaximumFileCount()
        {
            return OutputFormats.Length;
        }

        public bool HasMutuallyExclusiveBands()
        {
            foreach (GreyscaleChannel channel in Channels)
            {
                if (channel.HasMutuallyExclusiveBands(Channels))
                {
                    return true;
                }
            }
            return false;
        }

        public bool HasMutuallyExclusiveBands(GreyscaleChannel otherChannel)
        {
            return HasMutuallyExclusiveBands(new GreyscaleChannel[] { otherChannel });
        }

        public bool HasMutuallyExclusiveBands(GreyscaleChannel[] otherChannels)
        {
            if (HasMutuallyExclusiveBands())
            {
                return true;
            }
            foreach (GreyscaleChannel channel in Channels)
            {
                if (channel.HasMutuallyExclusiveBands(otherChannels))
                {
                    return true;
                }
            }
            return false;
        }

        public bool HasMutuallyExclusiveBands(MultiChannelImage mci)
        {
            if (HasMutuallyExclusiveBands())
            {
                return true;
            }
            foreach (GreyscaleChannel channel in Channels)
            {
                if (channel.HasMutuallyExclusiveBands(mci.Channels))
                {
                    return true;
                }
            }
            return false;
        }

        //public bool UseAsLumaChannel()
        //{
        //    foreach (GreyscaleChannel channel in Channels)
        //    {
        //        if (!channel.UseAsLumaBand)
        //        {
        //            return false;
        //        }
        //    }
        //    return true;
        //}

        //public bool UseAsThreeChannelChannel()
        //{
        //    foreach (SpectralBand band in SpectralBands)
        //    {
        //        if (!band.UseAsThreeChannelChannel)
        //        {
        //            return false;
        //        }
        //    }
        //    return true;
        //}

        public virtual bool ContainsDisallowedChannels()
        {
            throw new Exception("This method is not implemented in this class.");
        }
    }
}
