// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class NotEnoughSourceSpectralBandsException : Exception
    {
        protected string _BaseDirectory;
        protected int _BandsInDirectory;
        protected int _RequiredBandCount;

        #region Properties

        public string BaseDirectory
        {
            get
            {
                return _BaseDirectory;
            }
        }

        public int BandsInDirectory
        {
            get
            {
                return _BandsInDirectory;
            }
        }

        public int RequiredBandCount
        {
            get
            {
                return _RequiredBandCount;
            }
        }

        #endregion

        public NotEnoughSourceSpectralBandsException(string baseDirectory, int bandsPresent, int bandsNeeded)
            : base(GetMessage(baseDirectory, bandsPresent, bandsNeeded))
        {
            _BaseDirectory = baseDirectory;
            _BandsInDirectory = bandsPresent;
            _RequiredBandCount = bandsNeeded;
        }

        protected static string GetMessage(string baseDirectory, int bandsPresent, int bandsNeeded)
        {
            return "Not enough source spectral bands were found in the directory '" + baseDirectory +
                    "' to allow processing. " + bandsPresent.ToString() +
                    " bands were found in source files, but " + bandsNeeded.ToString() +
                    " bands are required using the current configuration. If this directory should be processed, " +
                    "verify that the correct input configuration is in use and the source files are named correctly.";
        }
    }
}
