// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace BenLincoln.DaVinci
{
    //public enum NormalizationMode
    //{
    //    None,
    //    Standard,
    //    Pushover,           // reserved for future use
    //    MilqueToast,        // reserved for future use
    //    PassiveAggressive,  // reserved for future use
    //    ArmchairQuarterback,// reserved for future use
    //    Confident,
    //    Assertive,          // reserved for future use
    //    RattlingSaber,      // reserved for future use
    //    Aggressive,
    //    Militant,           
    //    Imperialist,        // reserved for future use
    //    NoMercy,            // reserved for future use
    //    NoRemorse,          
    //    ResistanceIsFutile, // reserved for future use
    //    TheWeakShallPerish, // reserved for future use
    //    ScourgeOfAllLife    // reserved for future use
    //}

    public class OutputImage
    {
        protected const string _DeallocateObjectScriptCode = " = 0";

        protected OutputParameterSet[] _OutputFormats;

        protected bool _WillOverwriteExistingData;
        protected bool _WillBeCreatedByScript;

        protected int _FileCountWillBeWritten;
        protected int _FileCountWillBeSkipped;

        protected bool _IncludeInSecondOrderImages;
        protected bool _UseInStandardLumaColourImages;

        protected string _OutputDirectory;

        protected string _UserDefinedName;

        protected string _ImageName;

        protected string _NamePrefix;
        protected string _NameComponentSeparator;
        protected string _NameSuffix;

        //protected NormalizationMode _OutputNormalization;
        protected TransformationProfileCollection _InputTransformations;
        protected TransformationProfileCollection _OutputTransformations;

        protected bool _UseAsLuminanceChannel;

        // which data organization mode to have DaVinci use when reading in and writing files
        protected const string _DataOrganizationMode = "bip";

        protected const string _CastOnLoad = "tofloat";

        //protected const string _AggressiveNormalizationParameters = "warpFieldGeometry = linear, pi = 3.7, acceleration = 182.7";
        //protected const string _AggressiveNormalizationParameters = "";
        //protected const string _AggressiveNormalizationScriptCode = "anormalize(%OBJECT%)";
        ////protected const string _ConfidentNormalizationParameters = "backlash = 5, threshold = 0.001";
        //protected const string _ConfidentNormalizationScriptCode = "anormalize(%OBJECT%, backlash = 1, threshold = 0.001)";
        ////protected const string _NoRemorseNormalizationParameters = "steps = 100000, backlash = 1, threshold = 0.01, threshrepeat = 3, searchwidth = 5000";
        ////protected const string _NoRemorseNormalizationScriptCode = "anormalize(anormalize(%OBJECT%, steps = 100000, backlash = 1, threshold = 0.01, threshrepeat = 3, searchwidth = 5000), backlash = 1, threshold = 0.01)";
        //protected const string _NoRemorseNormalizationScriptCode = "anormalize(anormalize(%OBJECT%, steps = 100000, backlash = 1, threshold = 0.01, threshrepeat = 3, searchwidth = 5000), steps = 500, threshold = 0.02, threshrepeat = 3, backlash = 0, searchwidth = 25)";
        //protected const string _MilitantNormalizationScriptCode = "anormalize(%OBJECT%, steps = 500, threshold = 0.02, threshrepeat = 3, backlash = 0, searchwidth = 25)";

        protected ImageCube _ParentCube;

        protected bool _IsCustomImage;

        #region Properties

        public bool IsCustomImage
        {
            get
            {
                return _IsCustomImage;
            }
            set
            {
                _IsCustomImage = value;
            }
        }

        public string ImageName
        {
            get
            {
                GenerateName();
                return _ImageName;
            }
            //set
            //{
            //    _ImageName = value;
            //}
        }

        public string UserDefinedName
        {
            get
            {
                return _UserDefinedName;
            }
            set
            {
                _UserDefinedName = value;
                GenerateName();
            }
        }

        public string NamePrefix
        {
            get
            {
                return _NamePrefix;
            }
            set
            {
                _NamePrefix = value;
                GenerateName();
            }
        }

        public string NameComponentSeparator
        {
            get
            {
                return _NameComponentSeparator;
            }
            set
            {
                _NameComponentSeparator = value;
                GenerateName();
            }
        }

        public string NameSuffix
        {
            get
            {
                return _NameSuffix;
            }
            set
            {
                _NameSuffix = value;
                GenerateName();
            }
        }

        /// <summary>
        /// This read-only value represents whether or not running the script which represents
        /// the image will overwrite data which already exists on disk.
        /// </summary>
        public bool WillOverwriteExistingData
        {
            get
            {
                return _WillOverwriteExistingData;
            }
        }

        /// <summary>
        /// This read-only value represents whether or not running the script which represents
        /// the image will create a file
        /// </summary>
        public bool WillBeCreatedByScript
        {
            get
            {
                return _WillBeCreatedByScript;
            }
        }

        public OutputParameterSet[] OutputFormats
        {
            get
            {
                return _OutputFormats;
            }
            set
            {
                _OutputFormats = value;
            }
        }

        public int FileCountWillBeWritten
        {
            get
            {
                return _FileCountWillBeWritten;
            }
        }

        public int FileCountWillBeSkipped
        {
            get
            {
                return _FileCountWillBeSkipped;
            }
        }

        /// <summary>
        /// Controls whether or not the image will be used as the basis for second-order images
        /// (Luma/Colour, etc.)
        /// </summary>
        public bool IncludeInSecondOrderImages
        {
            get
            {
                return _IncludeInSecondOrderImages;
            }
            set
            {
                _IncludeInSecondOrderImages = value;
            }
        }

        public bool UseInStandardLumaColourImages
        {
            get
            {
                return _UseInStandardLumaColourImages;
            }
            set
            {
                _UseInStandardLumaColourImages = value;
            }
        }

        public string OutputDirectory
        {
            get
            {
                //return _OutputDirectory;
                //if (_OutputDirectory == null)
                //{
                //    throw new Exception("The OutputDirectory value for this image has not been set.");
                //}
                if (_OutputDirectory == "")
                {
                    return "";
                }
                if (_OutputDirectory.EndsWith(Path.DirectorySeparatorChar.ToString()))
                {
                    return _OutputDirectory;
                }
                return _OutputDirectory + Path.DirectorySeparatorChar;
            }
            set
            {
                _OutputDirectory = value;
            }
        }

        //public NormalizationMode OutputNormalization
        //{
        //    get
        //    {
        //        return _OutputNormalization;
        //    }
        //    set
        //    {
        //        _OutputNormalization = value;
        //    }
        //}

        public TransformationProfileCollection InputTransformations
        {
            get
            {
                return _InputTransformations;
            }
            set
            {
                _InputTransformations = value;
            }
        }

        public TransformationProfileCollection OutputTransformations
        {
            get
            {
                return _OutputTransformations;
            }
            set
            {
                _OutputTransformations = value;
            }
        }

        public bool UseAsLuminanceChannel
        {
            get
            {
                return _UseAsLuminanceChannel;
            }
            set
            {
                _UseAsLuminanceChannel = value;
            }
        }

        public ImageCube ParentCube
        {
            get
            {
                return _ParentCube;
            }
            set
            {
                _ParentCube = value;
            }
        }

        #endregion

        public OutputImage()
        {
            _WillOverwriteExistingData = false;
            _WillBeCreatedByScript = false;
            _OutputFormats = new OutputParameterSet[1];
            _OutputFormats[0] = new OutputParameterSet();
            _OutputDirectory = "";
            //_OutputNormalization = NormalizationMode.None;
            InputTransformations = new TransformationProfileCollection();
            OutputTransformations = new TransformationProfileCollection();
            _UseAsLuminanceChannel = true;
            _FileCountWillBeSkipped = 0;
            _FileCountWillBeWritten = 0;
            _NameComponentSeparator = "-";
            _NameSuffix = "";
            _UserDefinedName = "";
            _IsCustomImage = false;
        }

        ~OutputImage()
        {
            _ParentCube = null;
        }

        protected void GenerateName()
        {
            if (_UserDefinedName != "")
            {
                _ImageName = _UserDefinedName;
                return;
            }
            _GenerateName();
        }

        public static string WrapNameComponentIfComplex(string nameComponent)
        {
            bool wrap = false;
            Regex rEx = new Regex(@"[^A-Za-z0-9_()]");
            if (rEx.IsMatch(nameComponent))
            {
                return "(" + nameComponent + ")";
            }
            return nameComponent;
        }

        protected virtual void _GenerateName() { }

        public virtual string GetCreationScriptCode(string scriptTargetObjectName, bool regenerateExistingFiles)
        {
            return "# Error: Creation code is not implemented in this class ('" +
                this.GetType().ToString() + "'). This error should never be displayed." + Environment.NewLine;
        }

        public string GetReadScriptCode(string scriptTargetObjectName)
        {
            StringBuilder builder = new StringBuilder("# Read in the image '");
            builder.Append(GetName());
            builder.Append("' and place its contents into the object '");
            builder.Append(scriptTargetObjectName);
            builder.Append("'.");
            builder.Append(Environment.NewLine);
            return _GetReadScriptCode(builder.ToString(), scriptTargetObjectName);
        }

        public virtual string GetName()
        {
            throw new Exception("This method is not implemented in this class.");
        }

        protected virtual string _GetReadScriptCode(string scriptPrepend, string scriptTargetObjectName)
        {
            throw new Exception("This method is not implemented in this class.");
        }

        public virtual int GetMaximumFileCount()
        {
            throw new Exception("This method is not implemented in this class.");
        }


        public static string GetDeallocationScriptCode(string scriptTargetObjectName)
        {
            return scriptTargetObjectName + _DeallocateObjectScriptCode + Environment.NewLine;
        }

        public static string GetConvertedOutputScriptCode(string scriptCopyObjectName, string scriptTargetObjectName, OutputParameterSet outputFormat)
        {
            StringBuilder builder = new StringBuilder(scriptCopyObjectName);
            builder.Append(" = ");
            switch (outputFormat.BitsPerChannel)
            {
                case BitsPerChannel.EightByte:
                    builder.Append("tobyte(");
                    break;
                case BitsPerChannel.SixteenShort:
                    builder.Append("toshort(");
                    break;
                case BitsPerChannel.ThirtyTwoFloat:
                    builder.Append("tofloat(");
                    break;
                case BitsPerChannel.SixtyFourDouble:
                    builder.Append("todouble(");
                    break;
                default:
                    throw new ArgumentException("This image's output bits per channel value is set to '" + outputFormat.BitsPerChannel.ToString() + "', which is not currently supported.");
                    break;
            }
            builder.Append(scriptTargetObjectName);
            builder.Append(")");
            builder.Append(Environment.NewLine);
            return builder.ToString();
        }

        public string GetImageFileWriteCode(string scriptTargetObjectName, OutputParameterSet[] outputFormats, string baseFilePath, bool regenerateExistingFiles)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(OutputTransformations.GetWrappedLine(scriptTargetObjectName));

            int imageNum = 0;
            foreach (OutputParameterSet outputFormat in outputFormats)
            {
                if (baseFilePath.Trim() == "")
                {
                    throw new Exception("Error: an empty string was generated as the base path for the image '" + ImageName + "'. This indicates a bug in the program.");
                }
                string outputFilePath = baseFilePath + OutputParameterSet.ImageFormatToFileExtension(outputFormat.FileFormat);
                bool generateFile = true;
                if (File.Exists(outputFilePath))
                {
                    if (regenerateExistingFiles)
                    {
                        builder.Append("# Warning: the file '" + outputFilePath + "' already exists, but the option to overwrite existing files is enabled.");
                        builder.Append(Environment.NewLine);
                        builder.Append("# The file will be overwritten.");
                        builder.Append(Environment.NewLine);
                    }
                    else
                    {
                        builder.Append("# Information: the file '" + outputFilePath + "' already exists, and the option to overwrite existing files is not enabled.");
                        builder.Append(Environment.NewLine);
                        builder.Append("# The file will not be overwritten.");
                        builder.Append(Environment.NewLine);
                        builder.Append("printf(\"Skipping existing file '" + outputFilePath + "'\\n\")");
                        builder.Append(Environment.NewLine);
                        generateFile = false;
                    }
                }
                else
                {
                    builder.Append("# Information: the file '" + outputFilePath + "' does not exist yet, and will be generated by the script.");
                    builder.Append(Environment.NewLine);
                }

                if (generateFile)
                {
                    _FileCountWillBeWritten++;
                    // create a temporary object so that each output variation will be produced from the 
                    // original (presumably highest-quality) original
                    string scriptCopyObjectName = scriptTargetObjectName + string.Format("_{0:00}", imageNum);
                    builder.Append(GetConvertedOutputScriptCode(scriptCopyObjectName, scriptTargetObjectName, outputFormat));
                    // TO DO: add scaling code

                    builder.Append("printf(\"Writing file '" + outputFilePath + "'\\n\")");
                    builder.Append(Environment.NewLine);

                    builder.Append("write(");
                    builder.Append(scriptCopyObjectName);
                    builder.Append(", \"");
                    //builder.Append(GetFilePath());
                    //builder.Append(baseFilePath);
                    //builder.Append(OutputParameterSet.ImageFormatToFileExtension(outputFormat.FileFormat));
                    builder.Append(outputFilePath);
                    builder.Append("\", type=");
                    builder.Append(OutputParameterSet.ImageFormatToDaVinciFileTypeString(outputFormat.FileFormat));
                    // overwriting existing files is protected against by other code
                    // setting force to 1 ensures that if any formats of the image need to be regenerated, they all are.
                    builder.Append(", force=1");
                    //builder.Append(", force=");
                    //if (outputFormat.RegenerateExistingFiles)
                    //{
                    //    builder.Append("1");
                    //}
                    //else
                    //{
                    //    builder.Append("0");
                    //}
                    builder.Append(")");
                    builder.Append(Environment.NewLine);
                    // de-allocate the temporary object
                    builder.Append(scriptCopyObjectName);
                    builder.Append(" = 0");
                    builder.Append(Environment.NewLine);
                    imageNum++;
                }
                else
                {
                    _FileCountWillBeSkipped++;
                }
            }
            return builder.ToString();
        }

        public static bool AllOutputFilesExist(string basePath, OutputParameterSet[] outputFormats)
        {
            bool result = true;
            foreach (OutputParameterSet outputFormat in outputFormats)
            {
                if (!File.Exists(basePath + OutputParameterSet.ImageFormatToFileExtension(outputFormat.FileFormat)))
                {
                    result = false;
                }
            }
            return result;
        }

        public static bool AnyOutputFilesExist(string basePath, OutputParameterSet[] outputFormats)
        {
            bool result = false;
            foreach (OutputParameterSet outputFormat in outputFormats)
            {
                if (File.Exists(basePath + OutputParameterSet.ImageFormatToFileExtension(outputFormat.FileFormat)))
                {
                    result = true;
                }
            }
            return result;
        }

        //public static NormalizationMode NormalizationModeFromString(string normalizationModeString)
        //{
        //    string nm = normalizationModeString.Trim().ToLower();

        //    switch (nm)
        //    {
        //        case "none":
        //            return NormalizationMode.None;
        //            break;
        //        case "standard":
        //            return NormalizationMode.Standard;
        //            break;
        //        case "pushover":
        //            throw new ArgumentException("The 'Pushover' normalization mode is reserved for future use.");
        //            break;
        //        case "milquetoast":
        //            throw new ArgumentException("The 'MilqueToast' normalization mode is reserved for future use.");
        //            break;
        //        case "passiveaggressive":
        //            throw new ArgumentException("The 'PassiveAggressive' normalization mode is reserved for future use.");
        //            break;
        //        case "armchairquarterback":
        //            throw new ArgumentException("The 'ArmchairQuarterback' normalization mode is reserved for future use.");
        //            break;
        //        case "confident":
        //            return NormalizationMode.Confident;
        //            break;
        //        case "assertive":
        //            throw new ArgumentException("The 'Assertive' normalization mode is reserved for future use.");
        //            break;
        //        case "rattlingsaber":
        //            throw new ArgumentException("The 'RattlingSaber' normalization mode is reserved for future use.");
        //            break;
        //        case "aggressive":
        //            return NormalizationMode.Aggressive;
        //            break;
        //        case "militant":
        //            return NormalizationMode.Militant;
        //            break;
        //        case "imperialist":
        //            throw new ArgumentException("The 'Imperialist' normalization mode is reserved for future use.");
        //            break;
        //        case "nomercy":
        //            throw new ArgumentException("The 'NoMercy' normalization mode is reserved for future use.");
        //            break;
        //        case "noremorse":
        //            return NormalizationMode.NoRemorse;
        //            break;
        //        case "resistanceisfutile":
        //            throw new ArgumentException("The 'ResistanceIsFutile' normalization mode is reserved for future use.");
        //            break;
        //        case "theweakshallperish":
        //            throw new ArgumentException("The 'TheWeakShallPerish' normalization mode is reserved for future use, at which time the weak shall indeed perish.");
        //            break;
        //        case "scourgeofalllife":
        //            throw new ArgumentException("The 'ScourgeOfAllLife' normalization mode is reserved for future use.");
        //            break;
        //    }

        //    throw new ArgumentException("The string '" + normalizationModeString + "' could not be converted into a recognized normalization mode.");
        //}

        //public static string GetNormalizationCodePre(NormalizationMode normalizationMethod)
        //{
        //    switch (normalizationMethod)
        //    {
        //        case NormalizationMode.Standard:
        //            return "normalize(";
        //            break;
        //        case NormalizationMode.Pushover:
        //            break;
        //        case NormalizationMode.MilqueToast:
        //            break;
        //        case NormalizationMode.PassiveAggressive:
        //            break;
        //        case NormalizationMode.ArmchairQuarterback:
        //            break;
        //        case NormalizationMode.Confident:
        //            return "anormalize(";
        //            break;
        //        case NormalizationMode.Assertive:
        //            break;
        //        case NormalizationMode.RattlingSaber:
        //            break;
        //        case NormalizationMode.Aggressive:
        //            return "anormalize(";
        //            break;
        //        case NormalizationMode.Militant:
        //            break;
        //        case NormalizationMode.Imperialist:
        //            break;
        //        case NormalizationMode.NoMercy:
        //            break;
        //        case NormalizationMode.NoRemorse:
        //            return "anormalize(";
        //            break;
        //        case NormalizationMode.ResistanceIsFutile:
        //            break;
        //        case NormalizationMode.TheWeakShallPerish:
        //            break;
        //        case NormalizationMode.ScourgeOfAllLife:
        //            break;
        //    }

        //    return "";
        //}

        //public static string GetNormalizationCodePost(NormalizationMode normalizationMethod)
        //{
        //    StringBuilder builder = new StringBuilder("");

        //    switch (normalizationMethod)
        //    {
        //        case NormalizationMode.Standard:
        //            builder.Append(")");
        //            break;
        //        case NormalizationMode.Pushover:
        //            break;
        //        case NormalizationMode.MilqueToast:
        //            break;
        //        case NormalizationMode.PassiveAggressive:
        //            break;
        //        case NormalizationMode.ArmchairQuarterback:
        //            break;
        //        case NormalizationMode.Confident:
        //            if (_ConfidentNormalizationParameters.Trim() != "")
        //            {
        //                builder.Append(", ");
        //                builder.Append(_ConfidentNormalizationParameters);
        //            }
        //            builder.Append(")");
        //            break;
        //        case NormalizationMode.Assertive:
        //            break;
        //        case NormalizationMode.RattlingSaber:
        //            break;
        //        case NormalizationMode.Aggressive:
        //            if (_AggressiveNormalizationParameters.Trim() != "")
        //            {
        //                builder.Append(", ");
        //                builder.Append(_AggressiveNormalizationParameters);
        //            }
        //            builder.Append(")");
        //            break;
        //        case NormalizationMode.Militant:
        //            break;
        //        case NormalizationMode.Imperialist:
        //            break;
        //        case NormalizationMode.NoMercy:
        //            break;
        //        case NormalizationMode.NoRemorse:
        //            if (_NoRemorseNormalizationParameters.Trim() != "")
        //            {
        //                builder.Append(", ");
        //                builder.Append(_NoRemorseNormalizationParameters);
        //            }
        //            builder.Append(")");
        //            break;
        //        case NormalizationMode.ResistanceIsFutile:
        //            break;
        //        case NormalizationMode.TheWeakShallPerish:
        //            break;
        //        case NormalizationMode.ScourgeOfAllLife:
        //            break;
        //    }


        //    return builder.ToString();
        //}


        //public static string GetNormalizationCode(NormalizationMode normalizationMethod)
        //{
        //    switch (normalizationMethod)
        //    {
        //        case NormalizationMode.Standard:
        //            return "normalize(%OBJECT%)";
        //            break;
        //        case NormalizationMode.Pushover:
        //            break;
        //        case NormalizationMode.MilqueToast:
        //            break;
        //        case NormalizationMode.PassiveAggressive:
        //            break;
        //        case NormalizationMode.ArmchairQuarterback:
        //            break;
        //        case NormalizationMode.Confident:
        //            return _ConfidentNormalizationScriptCode;
        //            break;
        //        case NormalizationMode.Assertive:
        //            break;
        //        case NormalizationMode.RattlingSaber:
        //            break;
        //        case NormalizationMode.Aggressive:
        //            return _AggressiveNormalizationScriptCode;
        //            break;
        //        case NormalizationMode.Militant:
        //            return _MilitantNormalizationScriptCode;
        //            break;
        //        case NormalizationMode.Imperialist:
        //            break;
        //        case NormalizationMode.NoMercy:
        //            break;
        //        case NormalizationMode.NoRemorse:
        //            return _NoRemorseNormalizationScriptCode;
        //            break;
        //        case NormalizationMode.ResistanceIsFutile:
        //            break;
        //        case NormalizationMode.TheWeakShallPerish:
        //            break;
        //        case NormalizationMode.ScourgeOfAllLife:
        //            break;
        //    }


        //    return "%OBJECT%";
        //}

        //public static string GetFullNormalizationWrappedLine(string scriptObject, NormalizationMode normalizationMethod)
        //{
        //    if (normalizationMethod == NormalizationMode.None)
        //    {
        //        return "";
        //    }
        //    StringBuilder builder = new StringBuilder();
        //    builder.Append(scriptObject);
        //    builder.Append(" = ");
        //    builder.Append(WrapScriptObjectInNormalizationCode(scriptObject, normalizationMethod));
        //    return builder.ToString();
        //}

        //public static string WrapScriptObjectInNormalizationCode(string scriptObject, NormalizationMode normalizationMethod)
        //{
        //    StringBuilder builder = new StringBuilder();
        //    //builder.Append(scriptObject);
        //    //builder.Append(" = "); 
        //    if (normalizationMethod != NormalizationMode.None)
        //    {
        //        builder.Append(GetNormalizationCode(normalizationMethod));
        //        builder = builder.Replace("%OBJECT%", scriptObject);
        //        builder.Append(Environment.NewLine);
        //    }
        //    else
        //    {
        //        return scriptObject;
        //    }
        //    return builder.ToString();
        //}
    }
}
