// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class TMSBOutputConfiguration
    {
        protected bool _RegenerateExistingFiles;
        protected bool _RenameAllBandsFiles;
        protected bool _RenameRGBFiles;

        protected OutputParameterSet[] _OutputFormats;

        #region Properties

        public bool RegenerateExistingFiles
        {
            get
            {
                return _RegenerateExistingFiles;
            }
            set
            {
                _RegenerateExistingFiles = value;
            }
        }

        /// <summary>
        /// Controls whether or not the channels (and colour images) which contain all
        /// the spectral bands will be renamed from a list of bands to "All Bands"
        /// e.g. "All_Bands_(Avg)" instead of "NIR_R_G_B_UVA_(Avg)"
        /// </summary>
        public bool RenameAllBandsFiles
        {
            get
            {
                return _RenameAllBandsFiles;
            }
            set
            {
                _RenameAllBandsFiles = value;
            }
        }

        /// <summary>
        /// Controls whether or not the channels (and colour images) which contain 
        /// the bands corresponding to the colours humans perceive as "red", "green", 
        /// and "blue" will be renamed to remove the "FC-" prefix
        /// </summary>
        public bool RenameRGBFiles
        {
            get
            {
                return _RenameRGBFiles;
            }
            set
            {
                _RenameRGBFiles = value;
            }
        }

        public OutputParameterSet[] OutputFormats
        {
            get
            {
                return _OutputFormats;
            }
            set
            {
                _OutputFormats = value;
            }
        }

        #endregion

        public TMSBOutputConfiguration()
        {
            _RegenerateExistingFiles = false;
            _RenameAllBandsFiles = false;
            _RenameRGBFiles = false;
            _OutputFormats = new OutputParameterSet[0];
        }

        public void AddOutputFormat(ImageFormat fileFormat, BitsPerChannel bitsPerChannel, ScalingMode scalingMode, float scalingValue)
        {
            OutputParameterSet newFormat = new OutputParameterSet();
            newFormat.FileFormat = fileFormat;
            newFormat.BitsPerChannel = bitsPerChannel;
            newFormat.ScalingMode = scalingMode;
            newFormat.ScalingValue = scalingValue;

            AddOutputFormat(newFormat);
        }

        public void AddOutputFormat(OutputParameterSet newFormat)
        {
            Array.Resize(ref _OutputFormats, _OutputFormats.Length + 1);
            _OutputFormats[_OutputFormats.Length - 1] = newFormat;
            _OutputFormats[_OutputFormats.Length - 1].RegenerateExistingFiles = RegenerateExistingFiles;
        }
    }
}
