// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;

namespace BenLincoln.DaVinci
{
    public class TMSBOutputConfigurationFile : TMSBConfigurationFile
    {
        protected TMSBOutputConfiguration _OutputConfig;
        protected TransformationProfileCollection _TransformationProfiles;

        #region Properties

        public TMSBOutputConfiguration OutputConfig
        {
            get
            {
                return _OutputConfig;
            }
            set
            {
                _OutputConfig = value;
            }
        }

        #endregion

        protected enum ConfigProcessingState
        {
            Normal,
            FormatSection,
            Format
        }

        public TMSBOutputConfigurationFile(string configFileName)
            : this(configFileName, null)
        {
        }

        public TMSBOutputConfigurationFile(string configFileName, TransformationProfileCollection transformationProfiles)
            : base()
        {
            _OutputConfig = new TMSBOutputConfiguration();
            _TransformationProfiles = transformationProfiles;
            ReadData(configFileName);
        }

        protected override void ReadTypeSpecificData(Stream sourceStream)
        {
            XmlTextReader xReader = new XmlTextReader(sourceStream);
            //xReader.Settings.Schemas.Add(_FileSchema);

            string currentElementName = "";
            ConfigProcessingState currentState = ConfigProcessingState.Normal;
            OutputParameterSet outputFormat = new OutputParameterSet();
            while (xReader.Read())
            {
                switch (xReader.NodeType)
                {
                    case XmlNodeType.Element:
                        currentElementName = xReader.Name;
                        switch (currentState)
                        {
                            case ConfigProcessingState.Normal:
                                if (currentElementName == "OutputFormats")
                                {
                                    currentState = ConfigProcessingState.FormatSection;
                                }
                                break;
                            case ConfigProcessingState.FormatSection:
                                if (currentElementName == "OutputFormat")
                                {
                                    currentState = ConfigProcessingState.Format;
                                    outputFormat = new OutputParameterSet();
                                }
                                break;
                        }
                        break;
                    case XmlNodeType.EndElement:
                        currentElementName = xReader.Name;
                        switch (currentState)
                        {
                            case ConfigProcessingState.FormatSection:
                                if (currentElementName == "OutputFormats")
                                {
                                    currentState = ConfigProcessingState.Normal;
                                }
                                break;
                            case ConfigProcessingState.Format:
                                if (currentElementName == "OutputFormat")
                                {
                                    OutputConfig.AddOutputFormat(outputFormat);
                                    currentState = ConfigProcessingState.FormatSection;
                                }
                                break;
                        }
                        break;
                    case XmlNodeType.Text:
                        string currentText = xReader.Value.Trim();
                        switch (currentState)
                        {
                            case ConfigProcessingState.Normal:
                                switch (currentElementName)
                                {
                                    case "RegenerateExistingFile":
                                        OutputConfig.RegenerateExistingFiles = StringToBool(currentText);
                                        break;
                                    case "RenameAllBandsFiles":
                                        OutputConfig.RenameAllBandsFiles = StringToBool(currentText);
                                        break;
                                    case "RenameRGBFiles":
                                        OutputConfig.RenameRGBFiles = StringToBool(currentText);
                                        break;
                                }
                                break;
                            case ConfigProcessingState.Format:
                                switch (currentElementName)
                                {
                                    case "OutputFileType":
                                        outputFormat.FileFormat = OutputParameterSet.ImageFormatFromString(currentText);
                                        break;
                                    case "BitsPerChannel":
                                        outputFormat.BitsPerChannel = OutputParameterSet.BitsPerChannelFromString(currentText);
                                        break;
                                    case "ScaleMode":
                                        outputFormat.ScalingMode = OutputParameterSet.ScalingModeFromString(currentText);
                                        break;
                                    case "ScaleValue":
                                        outputFormat.ScalingValue = float.Parse(currentText);
                                        break;
                                }
                                break;
                        }
                        break;
                }
            }
        }

    }
}
