// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class TransformationProfile : IComparable
    {
        protected string _Name;
        protected string _Description;

        #region Properties

        /// <summary>
        /// A name which identifies the profile.
        /// E.g.: "Aggressive (NA0.5 - Independent)" or "Standard (NA0.5 - Linked)"
        /// </summary>
        public string Name
        {
            get
            {
                return _Name;
            }
            set
            {
                _Name = value;
            }
        }

        /// <summary>
        /// A text-based description of the profile.
        /// E.g.: "This profile is useful for data with a very gradual transition from the 'fruit' to the 'rind'"
        /// </summary>
        public string Description
        {
            get
            {
                return _Description;
            }
            set
            {
                _Description = value;
            }
        }

        #endregion

        public TransformationProfile()
        {
            Name = "";
            Description = "";
        }

        public int CompareTo(object obj)
        {
            if (obj is TransformationProfile)
            {
                TransformationProfile b2 = (TransformationProfile)obj;
                return Name.CompareTo(b2.Name);
            }
            else
            {
                return Name.CompareTo(obj.ToString());
            }
        }

        public string GetWrappedLine(string scriptObject)
        {
            return GetWrappedLine(scriptObject, scriptObject);
        }

        public string GetWrappedLine(string targetObject, string scriptObject)
        {
            string wrapped = WrapScriptObjectInTransformationCode(scriptObject);
            if (wrapped == targetObject)
            {
                return "";
            }
            else
            {
                return targetObject + " = " +
                    wrapped +
                    Environment.NewLine;
            }
        }

        public virtual string WrapScriptObjectInTransformationCode(string scriptObject)
        {
            throw new NotImplementedException("This method is not implemented in the base class.");
        }

        public virtual string GetProfileTypeName()
        {
            throw new NotImplementedException("This method is not implemented in the base class.");
        }

    }
}
